abstract class GameMode {
  void draw() {
  }
  void mouseClicked() {
  }
  void keyPressed() {
  }
  void keyReleased() {
  }
  void mousePressed() {
  }
  void mouseReleased() {
  }
}

//MODE with menu
class HomeMode extends GameMode {
  boolean showsCredits=false;
  int selection=1;

  void draw() { 
    image(menu[0], 0, 0, width, height);
    image(menu[6], 0, 0, width, height);
    image(menu[selection], 0, 0, width, height);
    if (showsCredits==true) {
      image(credits, 0, 0, width, height);
    }
  }

  void keyPressed() {
    if (showsCredits==true) {
      showsCredits=false;
    }

    //coordinate in menu
    if (keyCode==DOWN) {
      selection++;
      
      if (selection>5) {
        selection=1;
      }
    } else if (keyCode==UP) {
      selection--;
     
      if (selection<1) {
        selection=5;
      }

      //select a menu point
    } else if (keyCode==ENTER) {
      switch(selection) {
      case 1:
        gameMode=new GameLoadsMode();
        break;
      case 2:
        gameMode =new Handbook();
        break;
      case 3:
        gameMode=new CustomizeMode();
        break;
      case 4:
        showsCredits=true;
        break;
      case 5:
        exit();
        break;
      }
    }
  }
}

//loading Screen while Game is loading
class GameLoadsMode extends GameMode {
  int counter=0;

  void draw() {
    image(loadingScreen, 0, 0, width, height);
  }
  void keyReleased() {
    //start Game
    delay(300);
    gameMode = new UnicornGame();
  }
}

// MODE for Game Over
class GameEndsMode extends GameMode {
  int selection=1;

  GameEndsMode() {
    ingame.pause(); //AUDIO
    ingame.rewind(); //AUDIO
    nightSky_ingame.pause(); //AUDIO
    nightSky_ingame.rewind(); //AUDIO
    gameOver.play(); //AUDIO
    gameOver.rewind(); //AUDIO
  }

  void draw() { 

    image(gameOverScreen[0], 0, 0, width, height);

    //coordinate on Game Over Screen
    switch(selection) {
    case 1:
      image(gameOverScreen[selection], 0, 0, width, height);
      break;
    case 2:
      image(gameOverScreen[selection], 0, 0, width, height);
      break;
    }

    //display Score
    imageMode(CENTER);
    displayScore(); 
    imageMode(CORNER);
  }

  void keyPressed() {

    //coordinate in menu
    if (keyCode==LEFT) {
      selection--;
      if (selection<1) {
        selection=2;
      }
    }
    if (keyCode==RIGHT) {
      selection++;
      if (selection>2) {
        selection=1;
      }
    }

    //select a menu point
    if (keyCode==ENTER) {

      switch(selection) { 
      case 1:
        delay(500);
        gameMode=new UnicornGame();
        break;
      case 2:
        delay(500);
        gameMode=new HomeMode();
        homeScreen.rewind(); //AUDIO
        homeScreen.play(); //AUDIO
        homeScreen.loop(); //AUDIO
        break;
      }
    }
  }
}

//MODE for instruction handbook
class Handbook extends GameMode {
  int selection=0;

  Handbook() {
  }

  void draw() {
    image(handbook[selection], 0, 0, width, height);
  }

  void keyPressed() {
    //coordinate in handbook
    if (keyCode==LEFT) {
      selection--;

      if (selection<0) {
        selection=0;
      }
    }
    if (keyCode==RIGHT) {   
      selection++;
    }

    //get back to home menu
    if (selection>=7) {
      gameMode=new HomeMode();
    }
  }
}

//MODE to customize unicorn
class CustomizeMode extends GameMode {

  int selection1=indexPlayer1;
  int selection2=indexPlayer2;

  CustomizeMode() {
    //load unicorn Images in higher Resolution
    for (int i=0; i<unicornsBig.length; i++) {
      unicornsBig[i] = loadImage ("unicornsBig/"+i+".png");
    }
  }
  void draw() {
    image(customizeScreen[0], 0, 0, width, height);
    image(customizeScreen[1], 0, 0, width, height);


    //image unicorn in selected color
    imageMode(CENTER);
    image(unicornsBig[selection1], width/4+width/2, height/12+height/2, width/4.8, width/4.8);
    image(unicornsBig[selection2], width/4, height/12+height/2, width/4.8, width/4.8);
    imageMode(CORNER);
  }

  void keyPressed() {
    //coordinate in menu player1
    if (keyCode==RIGHT) {
      selection1++;
      if (selection1==selection2) {
        selection1++;
      }
      if (selection1>13) {  
        selection1=0;
      }
    }
    if (keyCode==LEFT) {
      selection1--;
      if (selection1==selection2) {
        selection1--;
      }
      if (selection1<0) {  
        selection1=13;
      }
    }

    //coordinate in menu player2
    if (keyCode=='d'||keyCode=='D') {
      selection2++;
      if (selection2==selection1) {
        selection2++;
      }
      if (selection2>13) {  
        selection2=0;
      }
    }
    if (keyCode=='a'||keyCode=='A') {
      selection2--;
      if (selection2==selection1) {
        selection2--;
      }
      if (selection2<0) {  
        selection2=13;
      }
    }

    //confirm selection and set player icons
    if (keyCode==ENTER) {
      indexPlayer1=selection1;
      indexPlayer2=selection2;

      player1Icon=unicornsSmall[indexPlayer1];
      player2Icon=unicornsSmall[indexPlayer2];

      gameMode=new HomeMode();
    }
  }
}

//image score in correct coordinates
void displayScore() {  
  drawScore(highScore, width/3.12, height/1.136, width/27.42);
  drawScore(scoreArray[0], width/5.2, height/5.14, width/29.54);
  drawScore(scoreArray[1], width/13.24, height/2.117, width/34.91);
  drawScore(scoreArray[2], width/4, height/1.66, width/42.6666);
}

//calculate correct Images and local coordinates for the score
void drawScore(int scoreValue, float x, float y, float size) {
  int onesPlace=(scoreValue%10)/1;
  int tensPlace=(scoreValue%100-onesPlace)/10;
  int hundredsPlace=(scoreValue%1000-onesPlace-tensPlace)/100;
  int thousandsPlace=(scoreValue%10000-onesPlace-tensPlace-hundredsPlace)/1000;
  int tenThousandsPlace=(scoreValue%100000-onesPlace-tensPlace-hundredsPlace-thousandsPlace)/10000;
  float x1 = x + (size-(size/4)) * 5;
  float x2 = x + (size-(size/4)) * 4;
  float x3 = x + (size-(size/4)) * 3;
  float x4 = x + (size-(size/4)) * 2;
  float x5 = x + (size-(size/4));
  float xm = x + (size-(size/5)) * 6;

  imageMode(CENTER);
  image(letters[6], xm, y, size, size);
  if (scoreValue<10) {
    image(numbers[onesPlace], x1, y, size, size);
  }
  if (scoreValue<100&&scoreValue>=10) { 
    image(numbers[onesPlace], x1, y, size, size);
    image(numbers[tensPlace], x2, y, size, size);
  }
  if (scoreValue<1000&&scoreValue>=100) { 
    image(numbers[onesPlace], x1, y, size, size);
    image(numbers[tensPlace], x2, y, size, size);
    image(numbers[hundredsPlace], x3, y, size, size);
  }
  if (scoreValue<10000&&scoreValue>=1000) { 
    image(numbers[onesPlace], x1, y, size, size);
    image(numbers[tensPlace], x2, y, size, size);
    image(numbers[hundredsPlace], x3, y, size, size);
    image(numbers[thousandsPlace], x4, y, size, size);
  }
  if (scoreValue<100000&&scoreValue>=10000) { 
    image(numbers[onesPlace], x1, y, size, size);
    image(numbers[tensPlace], x2, y, size, size);
    image(numbers[hundredsPlace], x3, y, size, size);
    image(numbers[thousandsPlace], x4, y, size, size);
    image(numbers[tenThousandsPlace], x5, y, size, size);
  }
  imageMode(CORNER);
}
