// Unicorns aka Player 1 and 2
class Unicorn extends SphericalGameObject {

  float mass; // in kg
  boolean isMoving=false; // checks if unicorn is moving
  PImage icon;
  float startSpeed=0;
  int counter=0;//respawn pulse counter
  int rainbowCounter=0;
  int animationCounter=0;
  int rainbowColor=0;


  Unicorn(GameWorld _gameWorld, float x, float y, PImage _icon) {
    super(_gameWorld, x, y, 0.17);
    mass = 0.1;
    this.icon=_icon;
  }

  void draw() {
    stroke(#785151);
    strokeWeight(0.01);

    imageMode(CENTER);

    if (gameWorld.saveRespawn==true&&gameWorld.rainbowMode==false) {
      //pulse unicorn while respawning
      counter++;
      //  println(counter); //debug
      if (counter<=60&&counter>40) {
        tint(255, 100);
        stroke(#770000, 50);
        if (this==gameWorld.player1) {
          line(gameWorld.player1.s.x, gameWorld.player1.s.y, gameWorld.player2.s.x, gameWorld.player2.s.y);
        }
        image(icon, s.x, s.y, radius, radius);
        noTint();
        if (counter>=60) {
          counter=0;
        }
      }
      if (counter<=40) {
        if (this==gameWorld.player1) {
          line(gameWorld.player1.s.x, gameWorld.player1.s.y, gameWorld.player2.s.x, gameWorld.player2.s.y);
        }
        image(icon, s.x, s.y, radius, radius);
      }
    } else if (gameWorld.rainbowMode==true) { //rainbow unicorns if rainbow mode is activated

      rainbowCounter++;
      //println(rainbowCounter); //Debug
      animationCounter=animationCounter+20;
      if (animationCounter>850) {
        animationCounter=0;
      }
      if (this==gameWorld.player1) {
        line(gameWorld.player1.s.x, gameWorld.player1.s.y, gameWorld.player2.s.x, gameWorld.player2.s.y);
      }
      image(rainbowUnicorn[animationCounter/100], s.x, s.y, radius, radius);
    } else {
      noTint();
      if (this==gameWorld.player1) {
        line(gameWorld.player1.s.x, gameWorld.player1.s.y, gameWorld.player2.s.x, gameWorld.player2.s.y);
      }
      image(icon, s.x, s.y, radius, radius);
    }
    imageMode(CORNER);
  }

  void move(int pressedKey) {

    //if no keyPressed: unicorn moves leftdown
    if (pressedKey == noPressed) {
      v.add(PVector.mult(new PVector(-1, 1), 0.001));
    } 

    //set unicorn speed
    if (gameWorld.rainbowMode==true) { //faster if rainbowMode is active
      startSpeed=0.02;
      gameWorld.speed=startSpeed;
    } else {
      startSpeed=0.012;
      gameWorld.speed=startSpeed;
      
    }
    //moves unicorns if wasd/ ULRD are pressed
    if (pressedKey == 'w'|| pressedKey == 'W') {
      v.add(PVector.mult(new PVector(0, 1), -gameWorld.speed));
    }
    if (pressedKey == 'a'|| pressedKey == 'A') {
      v.add(PVector.mult(new PVector(1, 0), -gameWorld.speed));
    }
    if (pressedKey == 's'|| pressedKey == 'S') {
      v.add(PVector.mult(new PVector(0, 1), gameWorld.speed));
    }
    if (pressedKey == 'd'|| pressedKey == 'D') {
      v.add(PVector.mult(new PVector(1, 0), gameWorld.speed));
    }
    if (pressedKey == UP) {
      v.add(PVector.mult(new PVector(0, 1), -gameWorld.speed));
    }
    if (pressedKey == LEFT) {
      v.add(PVector.mult(new PVector(1, 0), -gameWorld.speed));
    }
    if (pressedKey == RIGHT) {
      v.add(PVector.mult(new PVector(1, 0), gameWorld.speed));
    }
    if (pressedKey == DOWN) {
      v.add(PVector.mult(new PVector(0, 1), gameWorld.speed));
    }
  }


  //handle distance of Unicorns through springs 

  void applyForce(PVector force, float deltaTimeSeconds) {
    float dampingFactor = 0.1;
    PVector f = force;
    if (a.mag() >= 0.5) {
    a.mult(pow(dampingFactor, deltaTimeSeconds));
    } else {
      f.div(mass*2);
      a.add(f);
    }
  }

//credits: to prof. Fuchs (code example: marbles)
  void handleSphericalGameObjectCollision(SphericalGameObject sgo, 
    float deltaTimeSeconds) {

    /** Unicorn-Unicorn collisions */

    if (objectID >= sgo.objectID) { // make sure to check collisions only once 
      return;
    }
    if (! (sgo instanceof Unicorn)) { // we are only interested in Unicorn-Unicorn collisions
      return;
    }
    Unicorn u = (Unicorn) sgo;

    // set up the quadratic equation
    PVector x = PVector.sub(s, u.s);
    PVector y = PVector.sub(u.v, v);
    float z = radius-0.07 + u.radius-0.07;

    float a = PVector.dot(y, y);
    float b = 2 * PVector.dot(x, y);
    float c = PVector.dot(x, x) - z*z;

    float discriminant = b*b-4*a*c;

    float t; // we want to compute

    if (abs(a) < MIN_COLLISION_EPSILON) { // degenerate case: just a linear equation
      if (abs(b) < MIN_COLLISION_EPSILON) {
        // constant equation ? either true statement for all choices of t (constantly in touch) or false for any = either way, no collision for us to treat.
        return;
      }
      t = -c / b;
      if (t < 0) { // negative means: solution in the future - discard.
        return;
      }
      if (t >= deltaTimeSeconds) { // solution occurs too far in the past
        return;
      }
    } else { // quadratic equation

      if (discriminant < 0) {
        return; // no solutiuons
      }

      float sqd = sqrt(discriminant);

      // solutions (by design, t1 <= t2):

      float t1 = (-b - sqd ) / (2.0 * a);
      float t2 = (-b + sqd ) / (2.0 * a);

      if (t2 < 0) { // both solutions negative - both in the future - discard.
        return;
      }
      if (t1 >= deltaTimeSeconds) { // both solutions occur too far in the past
        return;
      }

      // we need the earlier solution that happened in this timestep.

      t = (t2 < deltaTimeSeconds) ? t2 : t1;
    }

    if (t >= deltaTimeSeconds) { // collision was in the past
      return;
    }

    // update velocities =======================================

    PVector vold1Vector = v.copy();
    PVector vold2Vector = u.v.copy();

    PVector s1coll = PVector.sub(s, PVector.mult(v, t));
    PVector s2coll = PVector.sub(u.s, PVector.mult(u.v, t));
    PVector d = PVector.sub(s2coll, s1coll).normalize();    

    float v1old = PVector.dot(v, d);
    float v2old = PVector.dot(u.v, d);

    float deltaV = 6.0 * (mass * v1old + u.mass * v2old) / (mass + u.mass);

    float v1new = deltaV - v1old;
    float v2new = deltaV - v2old;

    v.add(PVector.mult(d, v1new-v1old*2.0));
    u.v.add(PVector.mult(d, v2new-v2old*2.0));

    s.add(PVector.mult(vold1Vector, -t));
    s.add(PVector.mult(v, t));

    u.s.add(PVector.mult(vold2Vector, -t));
    u.s.add(PVector.mult(u.v, t));
  }

  void handleDistance(SphericalGameObject sgo, float deltaTimeSeconds) {

    if (objectID >= sgo.objectID) { // make sure to check collisions only once 
      return;
    }
    if (! (sgo instanceof Unicorn)) { // we are only interested in Unicorn-Unicorn collisions
      return;
    }    
    PVector distance = PVector.sub(s, sgo.s);
    float lengthUnicornDistance = distance.mag();
    if (lengthUnicornDistance > 0.5) {

      PVector vold1 = sgo.v.copy();

      sgo.v.y = -vold1.y; //* (-1.6);
      sgo.v.x = -vold1.x; //* (-1.6);

      sgo.s.add(PVector.mult(vold1, 0.5));
      sgo.s.add(PVector.mult(sgo.v, 0.5));

      PVector vold2 = v.copy();

      v.y = -vold2.y; //* (-1.6);
      v.x = -vold2.x; // * (-1.6);

      s.add(PVector.mult(vold2, 0.5));
      s.add(PVector.mult(v, 0.5));
    }
  }
}
