class UnicornGame extends GameMode {

  GameWorld gameWorld;
  UI ui;
  int noPressed;
  int ruckelCounter=0;
  float timeCounter=0;

  float gameSpeed=1;//counts the game Speed
  float heightCounter=0;//counts the height of the game
  float startSaveCounter=0;//counts length of save spawn at the start of the game
  boolean pauseGame=false; //checks if game is Paused
  boolean keyIsReleased=true; //checks if key is still Pressed

  int millisAtGameStart;


  // UI attributes ===================================================================

  // last time draw() was called - for animations
  int millisDrawWasLastCalled;


  // constructor and methods =========================================================

  UnicornGame() {
    millisDrawWasLastCalled = millis();  
    gameWorld = new GameWorld();
    ui=new UI(gameWorld);
    millisAtGameStart=millis();    
    homeScreen.pause(); //AUDIO
    homeScreen.rewind();//AUDIO
    ingame.play(); //AUDIO
    ingame.loop(); //AUDIO
    gameWorld.glitterScore=0;
  }

  //modify keyPressed and keyReleased
  void keyPressed() {
    if (keyCode>255) return; 
    gameWorld.keys[keyCode] = true;
    // System.out.println (keyCode); //debug
    if (key>255) return;
    gameWorld.keys[key] = true;
  }

  void keyReleased() {
    if (keyCode>255) return;
    gameWorld.keys[keyCode] = false;
    // System.out.println ("!" + keyCode); //debug
    if (key>255) return;
    gameWorld.keys[key] = false;
  }

  void draw() {
    pauseGame();
    int millisNow = millis();
    float timeElapsed = (millisNow - millisDrawWasLastCalled) / 1000.0; // since the last frame, in seconds.

    //makes sure that every TimeStep is the same
    if (timeElapsed>0.1) {
      println("ruckelt"); 
      timeElapsed=0.1;
      if (ruckelCounter<1) { //make sure that game starts at startSpeed, even if game start is delayed
        ruckelCounter++;
        millisAtGameStart=millis();
      }
    }
    if (pauseGame==false) {// normal game play

      startSaveCounter+=timeElapsed;
      updateSpeed(timeElapsed);

      if (startSaveCounter<2.5) {//saveRespawn at the beginning of the game
        gameWorld.saveRespawn=true;
      } else if (millis()-gameWorld.saveRespawnTime>3000&&(startSaveCounter>2.5)) {//ends every save Respawn
        gameWorld.saveRespawn=false;
      }


      //set GameWorld coordinates
      pushMatrix(); 

      scale( width/(float) gameWorld.worldWidthMeters, height/(float) gameWorld.worldHeightMeters);

      //if a key is pressed the Unicorn has to move
      if (gameWorld.keys['w'] || gameWorld.keys['W']) gameWorld.moveUnicorn('w');
      if (gameWorld.keys['a'] || gameWorld.keys ['A']) gameWorld.moveUnicorn('a');
      if (gameWorld.keys ['s'] || gameWorld.keys ['S']) gameWorld.moveUnicorn('s');
      if (gameWorld.keys ['d'] || gameWorld.keys ['D']) gameWorld.moveUnicorn('d');

      //if no key is pressed the Unicorn stops its movement
      if (gameWorld.keys['w'] == false || gameWorld.keys['W'] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys['a'] == false || gameWorld.keys ['A'] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys ['s'] == false || gameWorld.keys ['S'] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys ['d'] == false || gameWorld.keys ['D'] == false) gameWorld.moveUnicorn(noPressed);

      //if a key is pressed the Unicorn has to move
      if (gameWorld.keys[UP]) gameWorld.moveUnicorn(UP);
      if (gameWorld.keys[LEFT]) gameWorld.moveUnicorn(LEFT);
      if (gameWorld.keys [DOWN]) gameWorld.moveUnicorn(DOWN);
      if (gameWorld.keys [RIGHT]) gameWorld.moveUnicorn(RIGHT);

      //if no key is pressed the Unicorn stops its movement
      if (gameWorld.keys[UP] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys[LEFT] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys [DOWN] == false) gameWorld.moveUnicorn(noPressed);
      if (gameWorld.keys [RIGHT] == false) gameWorld.moveUnicorn(noPressed);

      //cheats just for fun
   //   if (gameWorld.keys ['R'] == true || gameWorld.keys ['r'] == true) gameWorld.cheatRainbowMode();

      // draw, in the Processing world, includes updating the game state - hence, animate all objects from draw()
      gameWorld.animate(timeElapsed);

      gameWorld.draw();
      ui.draw();

      popMatrix();
    } else if (pauseGame==true) {//pause Screen, game stops     
      if (gameWorld.rainbowMode==true) {//makes sure that rainbowCount stops while game is paused
        gameWorld.gamePausedTime+=(millisNow-millisDrawWasLastCalled);
      }
    }
    millisDrawWasLastCalled = millisNow;
  }

  //pause Modus
  void pauseGame() {
    if (keyIsReleased==true&&pauseGame==true) { 
      if (gameWorld.keys[82]) {//restart game
        gameMode=new GameLoadsMode();
      } 
      if (gameWorld.keys[80]) {//continue gameplay
        pauseGame=false;
        keyIsReleased=false;
        if (gameWorld.rainbowMode==true) {
          unicornMode.play();//AUDIO
        } else if (gameWorld.gameHeight<gameWorld.background.darkBlueHeight) {
          ingame.unmute();//AUDIO
        } else if (gameWorld.gameHeight>gameWorld.background.darkBlueHeight) {
          nightSky_ingame.unmute();//AUDIO
        }
      }
    } else if (keyIsReleased==true&&pauseGame==false) {
      if (gameWorld.keys[80]) {//pause gameplay
        pauseGame=true;
        image(pauseScreen, 0, 0, width, height);
        keyIsReleased=false;
        if (gameWorld.rainbowMode==true) {
          unicornMode.pause();//AUDIO
        } else if (gameWorld.gameHeight<gameWorld.background.darkBlueHeight) {
          ingame.mute();//AUDIO
        } else if (gameWorld.gameHeight>gameWorld.background.darkBlueHeight) {
          nightSky_ingame.mute();//AUDIO
        }
      }
    } 
    if (gameWorld.keys[80]==false) {
      keyIsReleased=true;
    }
  }

  void updateSpeed(float deltaTimeSeconds) {
    //update gameSpeed
    timeCounter=0.05;
    gameSpeed+=(timeCounter*deltaTimeSeconds);

    if (gameWorld.rainbowMode==false) {
      gameWorld.updateSpeed=gameSpeed;//set updateSpeed variable
      heightCounter+=(gameWorld.updateSpeed); //update Height counter
    } else if (gameWorld.rainbowMode==true) {
      gameWorld.updateSpeed=gameSpeed+gameWorld.rainbowSpeed;//set updateSpeed variable in rainbowMode
      heightCounter+=(gameWorld.updateSpeed);//update Height counter
    }

    if (gameWorld.updateSpeed<1) {//speed is at least 1
      gameWorld.updateSpeed=1;
    }
    //println(gameWorld.updateSpeed);//Debug
    // println(gameHeight);//Debug
    gameWorld.gameHeight=(int)(heightCounter/50); //set final game height
    // println(gameWorld.gameHeight); //debug
    // println(gameWorld.goodBadBubbleRatio);//debug
  }


  //method to get gameWorld coordinates from pixels
  PVector pixelsToGameWorldCoordinates(int x, int y) {
    return new PVector((x - width/2) 
      / (float) width * gameWorld.worldWidthMeters, 
      (y - height / 2)
      / (float) height * gameWorld.worldHeightMeters );
  }
}
